use iced::{Element, Length, Task, widget::text};

#[derive(Debug, Clone)]
enum Message {
    CreateLine(Option<usize>, Line, bool),
    ChangeLine(usize, Line),
}

mod line;
use line::Line;

#[derive(Default)]
pub struct GUI {
    lines: Vec<Line>,
    current_line: usize,
}

impl GUI {
    pub fn run(title: &'static str) -> Result<(), iced::Error> {
        iced::application(title, Self::update, Self::view)
            .settings(iced::Settings {
                default_font: iced::font::Font::MONOSPACE,
                antialiasing: true,
                ..Default::default()
            })
            .theme(|_| iced::Theme::Nord)
            .centered()
            .run_with(move || Self::new())
    }

    fn new() -> (Self, Task<Message>) {
        let s = Self {
            ..Default::default()
        };

        (
            s,
            Task::done(Message::CreateLine(
                None,
                Line::Text("Testline".to_owned()),
                true,
            )),
        )
    }

    fn update(&mut self, message: Message) -> Task<Message> {
        match message {
            Message::CreateLine(idx, line, focus) => {
                let idx = if let Some(idx) = idx {
                    idx
                } else {
                    self.lines.len()
                };
                self.lines.insert(idx, line);
                if focus {
                    self.current_line = idx;
                }
            }
            Message::ChangeLine(idx, text) => {
                let r = self.lines.get_mut(idx);
                if let Some(r) = r {
                    std::mem::replace(r, text);
                } else {
                    // TODO: Error message to user
                }
            }
        }

        Task::none()
    }

    fn view(&self) -> Element<Message> {
        iced::widget::Column::from_vec(
            self.lines
                .iter()
                .enumerate()
                .map(move |(i, line)| {
                    match line {
                        Line::Text(text) => iced::widget::text_input("", text)
                            .width(Length::Fill)
                            .on_input(move |s| Message::ChangeLine(i, Line::Text(s)))
                            .on_submit(Message::CreateLine(
                                Some(i + 1),
                                Line::Text("".to_owned()),
                                true,
                            )),
                    }
                    .into()
                })
                .collect::<Vec<Element<Message>>>(),
        )
        .width(Length::Fill)
        .height(Length::Fill)
        .into()
    }
}
